
/**************************************************************************
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*
*     (c) Copyright 2006 Xilinx, Inc.
*     All rights reserved.
*
**************************************************************************/



/**************************************************************************
* Filename:     TestApp_MemoryClear.c
*
* Description:
* This software application is used to clear a region of external memory.
* This program should be initialized to BRAM.  This program clears external
* memory by writing zeros to external memory locations. This prevents false
* ECC errors while executing a program in external memory by initializing 
* the ECC checkbits.
*
*
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -------------------------------------------------------
* 1.00  jel  7-31-06  Initial Release
*
*
*
*
*
******************************************************************************/

/***************************** Include Files *********************************/

#include "xparameters.h"
#include "xstatus.h"
#include "xio.h"

/************************** Constant Definitions *****************************/
#define MEMORY_SIZE 15000
#define MEMORY_STARTADDR 0x00000000

/************************** Function Prototypes ******************************/
XStatus ClearMemory();

/****************************************************************************
*
* This is main function that invokes the write to external memory function.
*
* @param    None
*
* @return   XST_SUCCESS to indicate success, otherwise XST_FAILURE.
*
* @note     None
*
******************************************************************************/


int main ()
{
  XStatus Status;
  Status = ClearMemory();
  if (Status != XST_SUCCESS)
    {
        return XST_FAILURE;
    }
}

/*****************************************************************************
*
* Writes to external memory based upon external memory's base address and
* memory size.
*
* @param    None
*
* @return   XST_SUCCESS to indicate success, otherwise XST_FAILURE.
*
* @note     None
*
******************************************************************************/

XStatus ClearMemory()
{
  int Index;
  XIo_Address Address;

  Address = MEMORY_STARTADDR;

  for(Index = 0; Index < MEMORY_SIZE; Index++)
    {
      XIo_Out32(Address, 0x00000000);
      Address = Address + 4;
    }
  return XST_SUCCESS;
}
